@@ -19,13 +19,8 @@
 
 #include "mozplugger.h"
 
-#define WINDOW ((Window) windata.window)
-
-#ifndef MIN
-#define MIN(X,Y) ((X)<(Y)?(X):(Y))
-#endif
-
 Display *display;
+int display_fd;
 char *displayname;
 NPWindow windata;
 int flags;
@@ -42,17 +37,6 @@ static Window victim;
 static int windows_found;
 static Window ignored[MAX_IGNORED];
 
-static int got_sigwinch;
-
-/*****************************************************************************
- * Functions below are called by the child process.
- *****************************************************************************/
-void sigwinch(int tmp)
-{
-     D("GOT SIGWINCH\n");
-     got_sigwinch = 1;
-}
-
 /*****************************************************************************
  * Called on X11 errors.
  *****************************************************************************/
@@ -70,7 +54,7 @@ int error_handler(Display *dpy, XErrorEvent *err)
 static void place_window()
 {
      int x;
-     Window parent = WINDOW;
+     Window parent = (Window) windata.window;
      int w = windata.width;
      int h = windata.height;
      XWMHints *leader_change;
@@ -78,9 +62,6 @@ static void place_window()
      D("Parent Window: %x\n", parent);
      D("Victim Window: %x\n", victim);
 
-     XWithdrawWindow(display, victim, XScreenNumberOfScreen (wattr.screen));
-     XSync(display, FALSE);
-
      D("Changing leader of window %x\n", victim);
      if ((leader_change = XGetWMHints(display, victim)))
      {
@@ -91,15 +72,12 @@ static void place_window()
      }
 
      D("Reparenting window %x into %x\n", victim, parent);
-     for (x = 0; x < 25; x++)
+     for (x = 0; x < REPARENT_REPEATS; x++)
      {
 	  XReparentWindow(display, victim, parent, 0, 0);
 	  XSync(display, FALSE);
      }
 
-     XMapWindow(display, victim);
-     XSync(display, FALSE);
-
      if (flags & H_FILL) {
 	  fill_window(display, victim, w, h);
      } else if (flags & H_MAXASPECT) {
@@ -274,18 +252,19 @@ static int event_swallow()
      {
 	  /* got a structure event */
 	  D("Got Structure Event of type %x\n", evt.type);
-	  if (evt.type == CreateNotify)
+	  if (evt.type == ReparentNotify)
 	  {
+		  
 	       /* looks promising, let's see if it's the one we're looking for */
-	       D("Found CreateNotify Event %x\n", evt.type);
+	       D("Found ReparentNotify Event %x\n", evt.type);
 
-	       if (!XGetClassHint(display, evt.xcreatewindow.window, &windowclass))
+	       if (!XGetClassHint(display, evt.xreparent.window, &windowclass))
 	       {
 		    D("Could Not get Class Hint, trying the next one");
 		    continue;
 	       }
 
-	       if (!XFetchName(display, evt.xcreatewindow.window, &windowname))
+	       if (!XFetchName(display, evt.xreparent.window, &windowname))
 	       {
 		    D("Could Not Get Window Name, trying the next one");
 		    continue;
@@ -300,7 +279,7 @@ static int event_swallow()
 			strncmp(victimname, windowclass.res_class, len) == 0 ||
 			strncmp(victimname, windowclass.res_name, len) == 0)
 		    {
-			 victim = evt.xcreatewindow.window;
+			 victim = evt.xreparent.window;
 			 found = 1;
 			 D("Found it!! Victim=%x\n", victim);
 			 D("Flags %x\n", flags);
@@ -328,7 +307,7 @@ static int init_swallow()
 
      D("display=%x\n", display);
 
-     if (!XGetWindowAttributes(display, WINDOW, &wattr))
+     if (!XGetWindowAttributes(display, (Window)windata.window, &wattr))
           return 1;
 
      D("rootwin=%x\n", wattr.root);
@@ -385,18 +364,39 @@ static void run_app(char **argv)
 }
 
 /*****************************************************************************
- * Handle the swallowing, the SIGWINCH signal, and wait.
+ Wait for an X event on the display.
+ *****************************************************************************/
+static void wait_event(int wait_time)
+{
+     fd_set fds;
+     struct timeval tv;
+     int ret;
+
+     tv.tv_sec = 0;
+     tv.tv_usec = wait_time;
+
+     FD_ZERO(&fds);
+     FD_SET(display_fd, &fds);
+     
+     D("select()\n");
+     if ((ret = select(display_fd+1, &fds, NULL, NULL, &tv)) == -1)
+	  exit(1);
+     D("ret=%d\n", ret);
+}
+
+/*****************************************************************************
+ * Handle the swallowing and wait.
  *****************************************************************************/
 static void handle_app(int pid)
 {
      int waitpid_done = 0;
      int swallow_done = 0;
      int status;
-     int tmp;
+     int timeout = time(NULL) + SWALLOW_MAX_LEN;
 
      if ((flags & H_SWALLOW) || (flags & H_EVENT_SWALLOW))
      {
-	  for (tmp = 0; tmp < SWALLOW_LOOPS; tmp++)
+	  while (time(NULL) < timeout)
 	  {
 	       if (!(flags & H_DAEMON))
 	       {
@@ -414,23 +414,19 @@ static void handle_app(int pid)
 
 	       if (swallow_done) break;
 
-	       usleep(SWALLOW_WAIT_TIME);
+	       if (flags & H_SWALLOW)
+		    usleep(SCAN_WAIT_TIME);
+	       else
+		    wait_event(EVENT_WAIT_TIME);
 	  }
 	  end_swallow();
      }
+
+     if (flags & H_DAEMON)
+	  exit(0);
      
      while (!waitpid_done)
      {
-	  if (got_sigwinch)
-	  {
-	       got_sigwinch = 0;
-	       if (swallow_done)
-		    place_window();
-	  }
-
-	  if (flags & H_DAEMON)
-	       exit(0);
-
 	  D("Waiting for godot... %d\n",pid);
 	  if (waitpid(pid, &status, 0) > 0)
 	       waitpid_done++;
@@ -462,8 +458,6 @@ int main(int argc, char **argv)
 	  exit(1);
      }
 
-     signal(SIGWINCH, sigwinch);
-
      sscanf(argv[1],"%d,%d,%lu,%d,%d,%d,%d",
 	    &flags,
 	    &repeats,
@@ -495,7 +489,7 @@ int main(int argc, char **argv)
 
 	  /* Expecting the application to loop */
 	  if (flags & H_LOOP)
-	       loops = 0x7fffffff;
+	       loops = MAXINT;
 
 	  /* Create command line */
 	  if (flags & H_CONTROLS)
@@ -521,6 +515,7 @@ int main(int argc, char **argv)
 	       if (!display)
 	       {
 		    display = XOpenDisplay(displayname);
+		    display_fd = ConnectionNumber(display);
 	       }
 
 	       if (init_swallow())
