@@ -23,10 +23,24 @@
 #endif
 
 /*****************************************************************************
- * Swallow loop parameters
+ * Active wait in useconds used with the flag swallow
  *****************************************************************************/
-#define SWALLOW_LOOPS     4000
-#define SWALLOW_WAIT_TIME 500
+#define SCAN_WAIT_TIME 500
+
+/*****************************************************************************
+ * Passive wait in useconds used with the flag event_swallow
+ *****************************************************************************/
+#define EVENT_WAIT_TIME (500 * 1000)
+
+/*****************************************************************************
+ * Maximum length of the swallow stage in seconds
+ *****************************************************************************/
+#define SWALLOW_MAX_LEN 60
+
+/*****************************************************************************
+ * How many times to reparent the window when found
+ *****************************************************************************/
+#define REPARENT_REPEATS 25
 
 /*****************************************************************************
  * Options
